﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading;
using System.Threading.Tasks;

namespace SequenceEx.Net.Http.OAuth
{
    // <summary> 
    /// Basic DelegatingHandler that creates an OAuth authorization header based on the OAuthBase 
    /// class downloaded from http://oauth.net 
    /// </summary> 
    public class OAuthMessageHandler : DelegatingHandler
    {
        private OAuthBase oAuthBase = new OAuthBase();
        private OAuthSettings oauthSettings;
        public OAuthMessageHandler(HttpMessageHandler innerHandler, IDictionary<string, object> oauthSettingParameters)
            : base(innerHandler)
        {
            if (oauthSettingParameters == null)
            {
                throw new ArgumentNullException("oauthSettingParameters");
            }

            this.oauthSettings = OAuthSettings.CreateFromDictionary(oauthSettingParameters);
        }

        protected override Task<HttpResponseMessage> SendAsync(HttpRequestMessage request, CancellationToken cancellationToken)
        {
            // Compute OAuth header  
            string normalizedUri;
            string normalizedParameters;
            string authHeader;

            string signature = this.oAuthBase.GenerateSignature(
                request.RequestUri,
                this.oauthSettings.ConsumerKey,
                this.oauthSettings.ConsumerSecret,
                this.oauthSettings.Token,
                this.oauthSettings.TokenSecret,
                request.Method.Method,
                this.oAuthBase.GenerateTimeStamp(),
                this.oAuthBase.GenerateNonce(),
                out normalizedUri,
                out normalizedParameters,
                out authHeader);

            request.Headers.Authorization = new AuthenticationHeaderValue("OAuth", authHeader);
            return base.SendAsync(request, cancellationToken);
        }

        private class OAuthSettings
        {
            internal string ConsumerKey
            {
                get;
                set;
            }

            internal string ConsumerSecret
            {
                get;
                set;
            }

            internal string Token
            {
                get;
                set;
            }

            internal string TokenSecret
            {
                get;
                set;
            }

            internal static OAuthSettings CreateFromDictionary(IDictionary<string, object> parameters)
            {
                OAuthSettings settings = new OAuthSettings();
                object value;

                if (parameters.TryGetValue("UseApplicationConfigurationFile", out value) && true.Equals(value))
                {
                    string appSettingsKey = null;

                    if (parameters.TryGetValue("AppSettingsKey", out value))
                    {
                        appSettingsKey = (string)value;
                    }

                    if (string.IsNullOrEmpty(appSettingsKey))
                    {
                        appSettingsKey = "OAuth";
                    }

                    
                    settings.ConsumerKey = ConfigurationManager.AppSettings[appSettingsKey + ":ConsumerKey"];
                    settings.ConsumerSecret = ConfigurationManager.AppSettings[appSettingsKey + ":ConsumerSecret"];
                    settings.Token = ConfigurationManager.AppSettings[appSettingsKey + ":Token"];
                    settings.TokenSecret = ConfigurationManager.AppSettings[appSettingsKey + ":TokenSecret"];
                }
                else
                {
                    if (parameters.TryGetValue("ConsumerKey", out value))
                    {
                        settings.ConsumerKey = (string)value;
                    }
                    if (parameters.TryGetValue("ConsumerSecret", out value))
                    {
                        settings.ConsumerSecret = (string)value;
                    }
                    if (parameters.TryGetValue("Token", out value))
                    {
                        settings.Token = (string)value;
                    }
                    if (parameters.TryGetValue("TokenSecret", out value))
                    {
                        settings.TokenSecret = (string)value;
                    }
                }

                return settings;
            }
        }
    }
}
